@@ -1,5 +1,7 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2000-2003 Marco Pesenti Gritti
+ *  Copyright © 2011 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,11 +32,11 @@
 #include "ephy-download.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
+#include "ephy-embed-type-builtins.h"
 #include "ephy-encodings.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-file-helpers.h"
 #include "ephy-history.h"
-#include "ephy-marshal.h"
 
 #include "ephy-print-utils.h"
 
@@ -57,7 +59,7 @@ struct _EphyEmbedShellPrivate
 	EphyAdBlockManager *adblock_manager;
 	GtkPageSetup *page_setup;
 	GtkPrintSettings *print_settings;
-	guint object_count;
+	EphyEmbedShellMode mode;
 	guint single_initialised : 1;
 };
 
@@ -66,18 +68,26 @@ enum
 	DOWNLOAD_ADDED,
 	DOWNLOAD_REMOVED,
 	PREPARE_CLOSE,
-	QUIT,
 	LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL];
 
+enum
+{
+	PROP_0,
+	PROP_MODE,
+	N_PROPERTIES
+};
+
+static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
+
 static void ephy_embed_shell_class_init	(EphyEmbedShellClass *klass);
 static void ephy_embed_shell_init	(EphyEmbedShell *shell);
 
 EphyEmbedShell *embed_shell = NULL;
 
-G_DEFINE_TYPE (EphyEmbedShell, ephy_embed_shell, G_TYPE_OBJECT)
+G_DEFINE_TYPE (EphyEmbedShell, ephy_embed_shell, GTK_TYPE_APPLICATION)
 
 static void
 ephy_embed_shell_dispose (GObject *object)
@@ -272,6 +282,42 @@ ephy_embed_shell_prepare_close (EphyEmbedShell *shell)
 	g_signal_emit (shell, signals[PREPARE_CLOSE], 0);
 }
 
+static void
+ephy_embed_shell_set_property (GObject *object,
+			       guint prop_id,
+			       const GValue *value,
+			       GParamSpec *pspec)
+{
+  EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (object);
+
+  switch (prop_id)
+  {
+  case PROP_MODE:
+	  embed_shell->priv->mode = g_value_get_enum (value);
+	  break;
+  default:
+	  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
+static void
+ephy_embed_shell_get_property (GObject *object,
+			       guint prop_id,
+			       GValue *value,
+			       GParamSpec *pspec)
+{
+  EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (object);
+
+  switch (prop_id)
+  {
+  case PROP_MODE:
+	  g_value_set_enum (value, embed_shell->priv->mode);
+	  break;
+  default:
+	  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
 static void
 ephy_embed_shell_init (EphyEmbedShell *shell)
 {
@@ -291,9 +337,24 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
 
 	object_class->dispose = ephy_embed_shell_dispose;
 	object_class->finalize = ephy_embed_shell_finalize;
+	object_class->set_property = ephy_embed_shell_set_property;
+	object_class->get_property = ephy_embed_shell_get_property;
 
 	klass->get_embed_single = impl_get_embed_single;
 
+	object_properties[PROP_MODE] =
+		g_param_spec_enum ("mode",
+				   "Mode",
+				   "The	 global mode for this instance of Epiphany .",
+				   EPHY_TYPE_EMBED_SHELL_MODE,
+				   EPHY_EMBED_SHELL_MODE_BROWSER,
+				   G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
+				   G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT_ONLY);
+	
+	g_object_class_install_properties (object_class,
+					   N_PROPERTIES,
+					   object_properties);
+
 /**
  * EphyEmbed::download-added:
  * @shell: the #EphyEmbedShell
@@ -345,25 +406,6 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
-/**
- * EphyEmbedShell::quit:
- * @shell: an #EphyEmbedShell
- * 
- * The ::quit is emitted when all windows (browser windows, popups,
- * download windows, etc) are closed and the @shell is ready to be
- * closed.
- *
- * Since: 2.30
- **/
-	signals[QUIT] =
-		g_signal_new ("quit",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE, 0);
-	
 	g_type_class_add_private (object_class, sizeof (EphyEmbedShellPrivate));
 }
 
@@ -605,21 +647,16 @@ ephy_embed_shell_remove_download (EphyEmbedShell *shell, EphyDownload *download)
 	g_signal_emit_by_name (shell, "download-removed", download, NULL);
 }
 
-
-static void
-object_notify_cb (EphyEmbedShell *shell, GObject *object)
+/**
+ * ephy_embed_shell_get_mode:
+ * @shell: an #EphyEmbedShell
+ * 
+ * Returns: the global mode of the @shell
+ **/
+EphyEmbedShellMode
+ephy_embed_shell_get_mode (EphyEmbedShell *shell)
 {
-	shell->priv->object_count--;
-	if (shell->priv->object_count == 0)
-		g_signal_emit (shell, signals[QUIT], 0);
-}
-
-void
-_ephy_embed_shell_track_object (EphyEmbedShell *shell, GObject *object)
-{
-	g_return_if_fail (EPHY_IS_EMBED_SHELL (shell));
-	g_return_if_fail (G_IS_OBJECT (object));
-
-	g_object_weak_ref (object, (GWeakNotify)object_notify_cb, shell);
-	shell->priv->object_count++;
+	g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), EPHY_EMBED_SHELL_MODE_BROWSER);
+	
+	return shell->priv->mode;
 }
